import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_types.l3_core_service import L3CoreService, L3CoreServiceType
from test.workflows import assert_complete, run_workflow


@pytest.mark.parametrize(
    "l3_core_service_type",
    [
        L3CoreServiceType.IMPORTED_GEANT_IP,
        L3CoreServiceType.IMPORTED_IAS,
        L3CoreServiceType.IMPORTED_LHCONE,
        L3CoreServiceType.IMPORTED_COPERNICUS,
    ],
)
@pytest.mark.workflow()
def test_import_l3_core_service_success(l3_core_service_subscription_factory, l3_core_service_type):
    imported_l3_core_service = l3_core_service_subscription_factory(l3_core_service_type=l3_core_service_type)
    result, _, _ = run_workflow("import_l3_core_service", [{"subscription_id": imported_l3_core_service}])
    subscription = L3CoreService.from_subscription(imported_l3_core_service)

    assert_complete(result)
    #  Remove the "IMPORTED_" prefix with ``[9:]``
    assert subscription.l3_core_service_type == L3CoreServiceType(l3_core_service_type.value[9:])
    assert subscription.status == SubscriptionLifecycle.ACTIVE
    assert subscription.insync is True
